#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Author: Tim Henderson
#Email: tim.tadh@gmail.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import
from six.moves import map
from six.moves import range

import copy
import itertools
import os
import sys
import random
import unittest
from random import randint, seed, shuffle

from zss import (
    simple_distance,
    Node,
)
from zss.compare import strdist

seed(os.urandom(15))

N = 3


def product(*args, **kwds):
    # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
    # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
    pools = list(map(tuple, args)) * kwds.get('repeat', 1)
    result = [[]]
    for pool in pools:
        result = [x+[y] for x in result for y in pool]
    for prod in result:
        yield tuple(prod)


if not hasattr(itertools, 'product'):
    setattr(itertools, 'product', product)


tree1_nodes = ['a','b','c','d','e','f']
def tree1():
    return (
        Node("f")
            .addkid(Node("d")
                .addkid(Node("a"))
                .addkid(Node("c")
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )


tree2_nodes = ['a','b','c','d','e','f']
def tree2():
    return (
        Node("a")
            .addkid(Node("c")
                .addkid(Node("d")
                    .addkid(Node("b"))
                    .addkid(Node("e"))))
            .addkid(Node("f"))
        )


tree3_nodes = ['a','b','c','d','e','f']
def tree3():
    return (
        Node("a")
            .addkid(Node("d")
                .addkid(Node("f"))
                .addkid(Node("c")
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )

tree4_nodes = ['q','b','c','d','e','f']
def tree4():
    return (
        Node("f")
            .addkid(Node("d")
                .addkid(Node("q"))
                .addkid(Node("c")
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )


def randtree(depth=2, alpha='abcdefghijklmnopqrstuvwxyz', repeat=2, width=2):
    labels = [''.join(x) for x in itertools.product(alpha, repeat=repeat)]
    shuffle(labels)
    labels = (x for x in labels)
    root = Node("root")
    p = [root]
    c = list()
    for x in range(depth-1):
        for y in p:
            for z in range(randint(1,1+width)):
                n = Node(next(labels))
                y.addkid(n)
                c.append(n)
        p = c
        c = list()
    return root


class TestTestNode(unittest.TestCase):

    def test_contains(self):
        root = tree1()
        self.assertTrue("a" in root)
        self.assertTrue("b" in root)
        self.assertTrue("c" in root)
        self.assertTrue("d" in root)
        self.assertTrue("e" in root)
        self.assertTrue("f" in root)
        self.assertFalse("q" in root)

    def test_get(self):
        root = tree1()
        self.assertEqual(root.get("a").label, "a")
        self.assertEqual(root.get("b").label, "b")
        self.assertEqual(root.get("c").label, "c")
        self.assertEqual(root.get("d").label, "d")
        self.assertEqual(root.get("e").label, "e")
        self.assertEqual(root.get("f").label, "f")

        self.assertNotEqual(root.get("a").label, "x")
        self.assertNotEqual(root.get("b").label, "x")
        self.assertNotEqual(root.get("c").label, "x")
        self.assertNotEqual(root.get("d").label, "x")
        self.assertNotEqual(root.get("e").label, "x")
        self.assertNotEqual(root.get("f").label, "x")

        self.assertEqual(root.get("x"), None)

    def test_iter(self):
        root = tree1()
        self.assertEqual(list(x.label for x in root.iter()), ['f','d','e','a','c','b'])


class TestCompare(unittest.TestCase):
    def test_distance(self):
        trees = itertools.product([tree1(), tree2(), tree3(), tree4()], repeat=2)
        for a,b in trees:
            ab = simple_distance(a,b)
            ba = simple_distance(b,a)
            #print '-----------------------------'
            #print a
            #print '------'
            #print b
            #print '------'
            #print ab, ba
            self.assertEqual(ab,ba)
            self.assertTrue((ab == 0 and a is b) or a is not b)
            #break
        trees = itertools.product([tree1(), tree2(), tree3(), tree4()], repeat=3)
        for a,b,c in trees:
            ab = simple_distance(a,b)
            bc = simple_distance(b,c)
            ac = simple_distance(a,c)
            self.assertTrue(ac <= ab + bc)
            #break

    #def test_randtree(self):
        #print randtree(5, repeat=3, width=2)

    def test_symmetry(self):
        trees = itertools.product((randtree(5, repeat=3, width=2) for x in range(N)), repeat=2)
        for a,b in trees:
            ab = simple_distance(a,b)
            ba = simple_distance(b,a)
            #print '-----------------------------'
            #print ab, ba
            self.assertEqual(ab, ba)

    def test_nondegenercy(self):
        trees = itertools.product((randtree(5, repeat=3, width=2) for x in range(N)), repeat=2)
        for a,b in trees:
            d = simple_distance(a,b)
            #print '-----------------------------'
            #print d, a is b
            self.assertTrue((d == 0 and a is b) or a is not b)

    def test_triangle_inequality(self):
        trees = itertools.product((randtree(5, repeat=3, width=2) for x in range(N)), (randtree(5, repeat=3, width=2) for x in range(N)), (randtree(5, repeat=3, width=2) for x in range(N)))
        for a,b,c in trees:
            #print '--------------------------------'
            ab = simple_distance(a,b)
            bc = simple_distance(b,c)
            ac = simple_distance(a,c)
            #print ab, bc, ac
            self.assertTrue(ac <= ab + bc)


    def test_labelchange(self):

        for A in (randtree(5, repeat=3, width=2) for x in range(N*4)):
            B = copy.deepcopy(A)
            node = random.choice([n for n in B.iter()])
            old_label = str(node.label)
            node.label = 'xty'
            assert simple_distance(A, B) == strdist(old_label, node.label)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        import cProfile
        cProfile.run('unittest.main()', 'profile')
    else:
        unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Author: Erick Fonseca
#Email: erickrfonseca@gmail.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import

from zss import simple_distance, Node, Operation

A = (
    Node("f")
        .addkid(Node("d")
                .addkid(Node("a"))
                .addkid(Node("c")
                        .addkid(Node("b"))))
        .addkid(Node("e"))
)
B = (
    Node("f")
        .addkid(Node("c")
                .addkid(Node("d")
                        .addkid(Node("a"))
                        .addkid(Node("b"))))
        .addkid(Node("e"))
)
C = (
    Node("a")
        .addkid(Node("c")
                .addkid(Node("d")
                        .addkid(Node("b"))
                        .addkid(Node("e"))))
        .addkid(Node("f"))
)


def all_equal(list1, list2):
    if len(list1) != len(list2):
        return False

    for item1, item2 in zip(list1, list2):
        if item1 != item2:
            return False

    return True


def test_ab():
    expected_ops = [Operation(Operation.match, Node("a"), Node("a")),
                    Operation(Operation.match, Node("b"), Node("b")),
                    Operation(Operation.remove, Node("c"), None),
                    Operation(Operation.match, Node("d"), Node("d")),
                    Operation(Operation.insert, None, Node("c")),
                    Operation(Operation.match, Node("e"), Node("e")),
                    Operation(Operation.match, Node("f"), Node("f"))]
    cost, ops = simple_distance(A, B, return_operations=True)
    assert ops == expected_ops


def test_ac():
    expected_ops = [Operation(Operation.remove, Node("a"), None),
                    Operation(Operation.match, Node("b"), Node("b")),
                    Operation(Operation.insert, None, Node("e")),
                    Operation(Operation.insert, None, Node("d")),
                    Operation(Operation.match, Node("c"), Node("c")),
                    Operation(Operation.remove, Node("d"), None),
                    Operation(Operation.update, Node("e"), Node("f")),
                    Operation(Operation.update, Node("f"), Node("a"))]
    cost, ops = simple_distance(A, C, return_operations=True)
    assert ops == expected_ops


def test_bc():
    expected_ops = [Operation(Operation.remove, Node("a"), None),
                    Operation(Operation.match, Node("b"), Node("b")),
                    Operation(Operation.insert, None, Node("e")),
                    Operation(Operation.match, Node("d"), Node("d")),
                    Operation(Operation.match, Node("c"), Node("c")),
                    Operation(Operation.update, Node("e"), Node("f")),
                    Operation(Operation.update, Node("f"), Node("a"))]
    cost, ops = simple_distance(B, C, return_operations=True)
    assert ops == expected_ops



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Author: Tim Henderson
#Email: tim.tadh@gmail.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import

from zss import (
    distance,
    simple_distance,
    Node,
)


try:
    from editdist import distance as strdist
except ImportError:
    def strdist(a, b):
        if a == b:
            return 0
        else:
            return 1

def weird_dist(A, B):
    return 10 * strdist(A, B)

class WeirdNode(object):

    def __init__(self, label):
        self.my_label = label
        self.my_children = list()

    @staticmethod
    def get_children(node):
        return node.my_children

    @staticmethod
    def get_label(node):
        return node.my_label

    def addkid(self, node, before=False):
        if before:  self.my_children.insert(0, node)
        else:   self.my_children.append(node)
        return self


def test_paper_tree():
    Node = WeirdNode
    A = (
      Node("f")
        .addkid(Node("d")
          .addkid(Node("a"))
          .addkid(Node("c")
            .addkid(Node("b"))
          )
        )
        .addkid(Node("e"))
    )
    B = (
      Node("f")
        .addkid(Node("c")
          .addkid(Node("d")
            .addkid(Node("a"))
            .addkid(Node("b"))
          )
        )
        .addkid(Node("e"))
    )
    #print A
    #print
    #print B
    dist = simple_distance(A, B, WeirdNode.get_children, WeirdNode.get_label,
        weird_dist)
    assert dist == 20


def test_rich_api():
    insert_cost = lambda node: 1
    remove_cost = lambda node: 1
    small_update_cost = lambda a, b: 1
    large_update_cost = lambda a, b: 3
    no_insert_cost = lambda node: 0

    A = Node('a')
    B = Node('b')
    # prefer update
    assert distance(
        A, B, Node.get_children, insert_cost, remove_cost,
        small_update_cost) == 1
    # prefer insert/remove
    assert distance(
        A, B, Node.get_children, insert_cost, remove_cost,
        large_update_cost) == 2

    C = Node('a', [Node('x')])
    assert (
        distance(
            A, C, Node.get_children, insert_cost, remove_cost,
            small_update_cost) >
        distance(
            A, C, Node.get_children, no_insert_cost, remove_cost,
            small_update_cost)
    )



from __future__ import absolute_import

from .compare import (
    distance,
    simple_distance,
)
from .compare import AnnotatedTree, Operation
from .simple_tree import Node

__all__ = ['distance', 'simple_distance', 'Node', 'AnnotatedTree', 'Operation']
__version__ = '1.2.0'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Authors: Tim Henderson and Steve Johnson
#Email: tim.tadh@gmail.com, steve@steveasleep.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import

from zss import (
    compare,
    Node,
)


def simple_trees():
    A = (
        Node("f")
            .addkid(Node("d")
                .addkid(Node("a"))
                .addkid(Node("c")
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )
    B = (
        Node("f")
            .addkid(Node("c")
                .addkid(Node("d")
                    .addkid(Node("a"))
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )
    return A, B

def test_nodes():
    A, B = [compare.AnnotatedTree(t, t.get_children) for t in simple_trees()]
    for i, nid in enumerate(reversed(A.ids)):
        assert nid == i
    for i, nid in enumerate(reversed(B.ids)):
        assert nid == i

def test_left_most_descendent():
    A, B = [compare.AnnotatedTree(t, t.get_children) for t in simple_trees()]
    assert A.lmds[0] == 0
    assert A.lmds[1] == 1
    assert A.lmds[2] == 1
    assert A.lmds[3] == 0
    assert A.lmds[4] == 4
    assert A.lmds[5] == 0

    assert B.lmds[0] == 0
    assert B.lmds[1] == 1
    assert B.lmds[2] == 0
    assert B.lmds[3] == 0
    assert B.lmds[4] == 4
    assert B.lmds[5] == 0

def test_keyroots():
    A, B = [compare.AnnotatedTree(t, t.get_children) for t in simple_trees()]
    assert A.keyroots == [2, 4, 5]
    assert B.keyroots == [1, 4, 5]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Authors: Tim Henderson and Steve Johnson
#Email: tim.tadh@gmail.com, steve@steveasleep.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import
from six.moves import range

import collections

try:
    import numpy as np
    zeros = np.zeros
except ImportError:
    def py_zeros(dim, pytype):
        assert len(dim) == 2
        return [[pytype() for y in range(dim[1])]
                for x in range(dim[0])]
    zeros = py_zeros

try:
    from editdist import distance as strdist
except ImportError:
    try:
        from editdistance import eval as strdist
    except ImportError:
        def strdist(a, b):
            if a == b:
                return 0
            else:
                return 1

from zss.simple_tree import Node


class AnnotatedTree(object):

    def __init__(self, root, get_children):
        self.get_children = get_children

        self.root = root
        self.nodes = list()  # a post-order enumeration of the nodes in the tree
        self.ids = list()    # a matching list of ids
        self.lmds = list()   # left most descendents
        self.keyroots = None
            # k and k' are nodes specified in the post-order enumeration.
            # keyroots = {k | there exists no k'>k such that lmd(k) == lmd(k')}
            # see paper for more on keyroots

        stack = list()
        pstack = list()
        stack.append((root, collections.deque()))
        j = 0
        while len(stack) > 0:
            n, anc = stack.pop()
            nid = j
            for c in self.get_children(n):
                a = collections.deque(anc)
                a.appendleft(nid)
                stack.append((c, a))
            pstack.append(((n, nid), anc))
            j += 1
        lmds = dict()
        keyroots = dict()
        i = 0
        while len(pstack) > 0:
            (n, nid), anc = pstack.pop()
            self.nodes.append(n)
            self.ids.append(nid)
            if not self.get_children(n):
                lmd = i
                for a in anc:
                    if a not in lmds: lmds[a] = i
                    else: break
            else:
                try: lmd = lmds[nid]
                except:
                    import pdb
                    pdb.set_trace()
            self.lmds.append(lmd)
            keyroots[lmd] = i
            i += 1
        self.keyroots = sorted(keyroots.values())


class Operation(object):
    """
    Dummy class for storing edit operations
    """
    remove = 0
    insert = 1
    update = 2
    match = 3

    def __init__(self, op, arg1=None, arg2=None):
        self.type = op
        self.arg1 = arg1
        self.arg2 = arg2

    def __repr__(self):
        if self.type == self.remove:
            return '<Operation Remove>'
        elif self.type == self.insert:
            return '<Operation Insert>'
        elif self.type == self.update:
            return '<Operation Update>'
        else:
            return '<Operation Match>'

    def __eq__(self, other):
        if other is None: return False
        if not isinstance(other, Operation):
            raise TypeError("Must compare against type Operation")
        return self.type == other.type and self.arg1 == other.arg1 and \
            self.arg2 == other.arg2


REMOVE = Operation.remove
INSERT = Operation.insert
UPDATE = Operation.update
MATCH = Operation.match


def simple_distance(A, B, get_children=Node.get_children,
        get_label=Node.get_label, label_dist=strdist, return_operations=False):
    """Computes the exact tree edit distance between trees A and B.

    Use this function if both of these things are true:

    * The cost to insert a node is equivalent to ``label_dist('', new_label)``
    * The cost to remove a node is equivalent to ``label_dist(new_label, '')``

    Otherwise, use :py:func:`zss.distance` instead.

    :param A: The root of a tree.
    :param B: The root of a tree.

    :param get_children:
        A function ``get_children(node) == [node children]``.  Defaults to
        :py:func:`zss.Node.get_children`.

    :param get_label:
        A function ``get_label(node) == 'node label'``.All labels are assumed
        to be strings at this time. Defaults to :py:func:`zss.Node.get_label`.

    :param label_dist:
        A function
        ``label_distance((get_label(node1), get_label(node2)) >= 0``.
        This function should take the output of ``get_label(node)`` and return
        an integer greater or equal to 0 representing how many edits to
        transform the label of ``node1`` into the label of ``node2``. By
        default, this is string edit distance (if available). 0 indicates that
        the labels are the same. A number N represent it takes N changes to
        transform one label into the other.

    :param return_operations: if True, return a tuple (cost, operations)
        where operations is a list of the operations to transform A into B.

    :return: An integer distance [0, inf+)
    """
    return distance(
        A, B, get_children,
        insert_cost=lambda node: label_dist('', get_label(node)),
        remove_cost=lambda node: label_dist(get_label(node), ''),
        update_cost=lambda a, b: label_dist(get_label(a), get_label(b)),
        return_operations=return_operations
    )


def distance(A, B, get_children, insert_cost, remove_cost, update_cost,
             return_operations=False):
    '''Computes the exact tree edit distance between trees A and B with a
    richer API than :py:func:`zss.simple_distance`.

    Use this function if either of these things are true:

    * The cost to insert a node is **not** equivalent to the cost of changing
      an empty node to have the new node's label
    * The cost to remove a node is **not** equivalent to the cost of changing
      it to a node with an empty label

    Otherwise, use :py:func:`zss.simple_distance`.

    :param A: The root of a tree.
    :param B: The root of a tree.

    :param get_children:
        A function ``get_children(node) == [node children]``.  Defaults to
        :py:func:`zss.Node.get_children`.

    :param insert_cost:
        A function ``insert_cost(node) == cost to insert node >= 0``.

    :param remove_cost:
        A function ``remove_cost(node) == cost to remove node >= 0``.

    :param update_cost:
        A function ``update_cost(a, b) == cost to change a into b >= 0``.

    :param return_operations: if True, return a tuple (cost, operations)
        where operations is a list of the operations to transform A into B.

    :return: An integer distance [0, inf+)
    '''
    A, B = AnnotatedTree(A, get_children), AnnotatedTree(B, get_children)
    size_a = len(A.nodes)
    size_b = len(B.nodes)
    treedists = zeros((size_a, size_b), float)
    operations = [[[] for _ in range(size_b)] for _ in range(size_a)]

    def treedist(i, j):
        Al = A.lmds
        Bl = B.lmds
        An = A.nodes
        Bn = B.nodes

        m = i - Al[i] + 2
        n = j - Bl[j] + 2
        fd = zeros((m,n), float)
        partial_ops = [[[] for _ in range(n)] for _ in range(m)]

        ioff = Al[i] - 1
        joff = Bl[j] - 1

        for x in range(1, m): # δ(l(i1)..i, θ) = δ(l(1i)..1-1, θ) + γ(v → λ)
            node = An[x+ioff]
            fd[x][0] = fd[x-1][0] + remove_cost(node)
            partial_ops[x][0].append(Operation(REMOVE, node))
        for y in range(1, n): # δ(θ, l(j1)..j) = δ(θ, l(j1)..j-1) + γ(λ → w)
            node = Bn[y+joff]
            fd[0][y] = fd[0][y-1] + insert_cost(node)
            partial_ops[0][y].append(Operation(INSERT, arg2=node))

        for x in range(1, m):  # the plus one is for the xrange impl
            for y in range(1, n):
                # x+ioff in the fd table corresponds to the same node as x in
                # the treedists table (same for y and y+joff)
                node1 = An[x+ioff]
                node2 = Bn[y+joff]
                # only need to check if x is an ancestor of i
                # and y is an ancestor of j
                if Al[i] == Al[x+ioff] and Bl[j] == Bl[y+joff]:
                    #                   +-
                    #                   | δ(l(i1)..i-1, l(j1)..j) + γ(v → λ)
                    # δ(F1 , F2 ) = min-+ δ(l(i1)..i , l(j1)..j-1) + γ(λ → w)
                    #                   | δ(l(i1)..i-1, l(j1)..j-1) + γ(v → w)
                    #                   +-
                    costs = [fd[x-1][y] + remove_cost(node1),
                             fd[x][y-1] + insert_cost(node2),
                             fd[x-1][y-1] + update_cost(node1, node2)]
                    fd[x][y] = min(costs)
                    min_index = costs.index(fd[x][y])

                    if min_index == 0:
                        op = Operation(REMOVE, node1)
                        partial_ops[x][y] = partial_ops[x-1][y] + [op]
                    elif min_index == 1:
                        op = Operation(INSERT, arg2=node2)
                        partial_ops[x][y] = partial_ops[x][y - 1] + [op]
                    else:
                        op_type = MATCH if fd[x][y] == fd[x-1][y-1] else UPDATE
                        op = Operation(op_type, node1, node2)
                        partial_ops[x][y] = partial_ops[x - 1][y - 1] + [op]

                    operations[x + ioff][y + joff] = partial_ops[x][y]
                    treedists[x+ioff][y+joff] = fd[x][y]
                else:
                    #                   +-
                    #                   | δ(l(i1)..i-1, l(j1)..j) + γ(v → λ)
                    # δ(F1 , F2 ) = min-+ δ(l(i1)..i , l(j1)..j-1) + γ(λ → w)
                    #                   | δ(l(i1)..l(i)-1, l(j1)..l(j)-1)
                    #                   |                     + treedist(i1,j1)
                    #                   +-
                    p = Al[x+ioff]-1-ioff
                    q = Bl[y+joff]-1-joff
                    costs = [fd[x-1][y] + remove_cost(node1),
                             fd[x][y-1] + insert_cost(node2),
                             fd[p][q] + treedists[x+ioff][y+joff]]
                    fd[x][y] = min(costs)
                    min_index = costs.index(fd[x][y])
                    if min_index == 0:
                        op = Operation(REMOVE, node1)
                        partial_ops[x][y] = partial_ops[x-1][y] + [op]
                    elif min_index == 1:
                        op = Operation(INSERT, arg2=node2)
                        partial_ops[x][y] = partial_ops[x][y-1] + [op]
                    else:
                        partial_ops[x][y] = partial_ops[p][q] + \
                            operations[x+ioff][y+joff]

    for i in A.keyroots:
        for j in B.keyroots:
            treedist(i, j)

    if return_operations:
        return treedists[-1][-1], operations[-1][-1]
    else:
        return treedists[-1][-1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Author: Tim Henderson
#Email: tim.tadh@gmail.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import

import collections


class Node(object):
    """
    A simple node object that can be used to construct trees to be used with
    :py:func:`zss.distance`.

    Example: ::

        Node("f")
            .addkid(Node("a")
                .addkid(Node("h"))
                .addkid(Node("c")
                    .addkid(Node("l"))))
            .addkid(Node("e"))
    """

    def __init__(self, label, children=None):
        self.label = label
        self.children = children or list()

    @staticmethod
    def get_children(node):
        """
        Default value of ``get_children`` argument of :py:func:`zss.distance`.

        :returns: ``self.children``.
        """
        return node.children

    @staticmethod
    def get_label(node):
        """
        Default value of ``get_label`` argument of :py:func:`zss.distance`.

        :returns: ``self.label``.
        """
        return node.label

    def addkid(self, node, before=False):
        """
        Add the given node as a child of this node.
        """
        if before:  self.children.insert(0, node)
        else:   self.children.append(node)
        return self

    def get(self, label):
        """:returns: Child with the given label."""
        if self.label == label: return self
        for c in self.children:
            if label in c: return c.get(label)

    def iter(self):
        """Iterate over this node and its children in a preorder traversal."""
        queue = collections.deque()
        queue.append(self)
        while len(queue) > 0:
            n = queue.popleft()
            for c in n.children: queue.append(c)
            yield n

    def __contains__(self, b):
        if isinstance(b, str) and self.label == b: return 1
        elif not isinstance(b, str) and self.label == b.label: return 1
        elif (isinstance(b, str) and self.label != b) or self.label != b.label:
            return sum(b in c for c in self.children)
        raise TypeError("Object %s is not of type str or Node" % repr(b))

    def __eq__(self, b):
        if b is None: return False
        if not isinstance(b, Node):
            raise TypeError("Must compare against type Node")
        return self.label == b.label

    def __ne__(self, b):
        return not self.__eq__(b)

    def __repr__(self):
        return super(Node, self).__repr__()[:-1] + " %s>" % self.label

    def __str__(self):
        s = "%d:%s" % (len(self.children), self.label)
        s = '\n'.join([s]+[str(c) for c in self.children])
        return s



from distutils.core import setup

setup(name='zss',
      version='1.2.0',
      description='Tree edit distance using the Zhang Shasha algorithm',
      author='Tim Henderson',
      author_email='tim.tadh@gmail.com',
      url='https://www.github.com/timtadh/zhang-shasha',
      packages=['zss'],
      install_requires=['six'],
      requires=['editdistance', 'numpy', 'six'],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#Author: Tim Henderson
#Email: tim.tadh@gmail.com
#For licensing see the LICENSE file in the top level directory.

from __future__ import absolute_import
from __future__ import print_function

import os
from random import seed

from zss.compare import (
    simple_distance,
    distance,
    strdist,
    Node,
)


seed(os.urandom(15))


def test_empty_tree_distance():
    assert simple_distance(Node(''), Node('')) == 0
    assert simple_distance(Node('a'), Node('')) == 1
    assert simple_distance(Node(''), Node('b')) == 1


def test_paper_tree():
    A = (
      Node("f")
        .addkid(Node("d")
          .addkid(Node("a"))
          .addkid(Node("c")
            .addkid(Node("b"))
          )
        )
        .addkid(Node("e"))
    )
    B = (
      Node("f")
        .addkid(Node("c")
          .addkid(Node("d")
            .addkid(Node("a"))
            .addkid(Node("b"))
          )
        )
        .addkid(Node("e"))
    )
    #print A
    #print
    #print B
    dist = simple_distance(A,B)
    assert dist == 2


def test_simplelabelchange():
    A = (
        Node("f")
            .addkid(Node("a")
                .addkid(Node("h"))
                .addkid(Node("c")
                    .addkid(Node("l"))))
            .addkid(Node("e"))
        )
    B = (
        Node("f")
            .addkid(Node("a")
                .addkid(Node("d"))
                .addkid(Node("r")
                    .addkid(Node("b"))))
            .addkid(Node("e"))
        )
    dist = simple_distance(A,B)
    print(dist)
    assert dist == 3
    #print 'distance', d


def test_incorrect_behavior_regression():
    A = (
     Node("a")
       .addkid(Node("b")
         .addkid(Node("x"))
         .addkid(Node("y"))
       )
     )
    B = (
     Node("a")
       .addkid(Node("x"))
       .addkid(Node("b")
         .addkid(Node("y"))
       )
     )
    dist = simple_distance(A, B)
    print(dist)
    assert dist == 2

def test_dict():
    A = {
        'name': 'tree', 
        'children': [
            {
                'name': 'child 1'
            },
            {
                'name': 'child 2'  
            }
            ]}
    B = {
        'name': 'tree', 
        'children': [
            {
                'name': 'child 1'
            },
            {
                'name': 'child Z'  
            }
            ]}
    dist = simple_distance(
        A,
        B,
        get_children=lambda x: x['children'] if 'children' in x else [],
        get_label=lambda x: x['name'],
        label_dist=lambda x,y: 1 if x!=y else 0
    )
    assert dist == 1

def test_wrong_index_regression():
    A = Node('r')
    B = (
        Node("a")
          .addkid(Node("b")
            .addkid(Node("c"))
            .addkid(Node("d"))
          )
          .addkid(Node("e")
            .addkid(Node("f"))
            .addkid(Node("g"))
          )
     )
    def insert(n):
        return ord(n.label[0])
    def remove(n):
        return ord(n.label[0])
    def update(a,b):
        return 10000*strdist(a.label, b.label)
    dist = distance(A, B,
        get_children=(lambda n: n.children),
        insert_cost=insert,
        remove_cost=remove,
        update_cost=update,
    )
    assert dist == 814

